% CDH Painkiller - newp roduct entry problem
% Solve using Stackelberg strategy with MPEC formulation
clear all
clc
format compact

global I J N chi v0
global Z y vv b w t
global p_init p_solu Z PL PU

load pzsol_stkg_true.mat
TolOverall=1e-9; % true: 1e-15
t=1e-10;

%I   subjects or consumers / 1 % 30 /,
%J   brand names 1
%N   coordinates to describe brands / asp, asub, caff, aing /;
% new product is J=15
I=30; J=15; N=4;

%chi randomness increases as chi decreases   / 3 /,
%Kdemand demand for no drug,
%v0=K  constant term in for no purchase option  / 1 /;
chi=3; v0=1;

%x(J,N) brand J's amount of N,
%y(I,N) subject I's preference for N,
%vv(I)  subject I's importance weight for every N,
%w(I)   price attributes,
%b(I)   constant term in DU eqn,
%C(J)   average cost of production,
%p_init(J) initial prices for brand J,
%p_solu(J) solution from Choi-DeSarbo-Harker paper,
%demand(J) computed demand for brand J,
% DU(I,J) matrix in disutility function;


% get data in choi.dat
init_choi_data;
C=sum(Z(:,1)*10*0.3+Z(:,2)*10*0.8+Z(:,3)*10*0.4+Z(:,4)*10*0.2,2);
brand=['A  ';'B  ';'C  ';'D  ';'E  ';...
       'F  ';'G  ';'H  ';'I  ';'J  ';...
       'K  ';'L  ';'M  ';'N  ';'New'];
   
% pz size=47 - here is the data structure
% p=pz(1:15);     % #15 is the price of the new product (14 competitors) (size=15)
% znew=pz(16:19); % the new product attributes (size=4)
% muPU=pz(20:33); % Lagrange multipliers for followers' price upper bounds (size=14)
% muPL=pz(34:47); % Lagrange multipliers for followers' price lower bounds (size=14)

% lower and upper bounds
%   p1-14  p   pnew    znew    mu1-28  
PL=C; PU=3*C;
lb=[PL;  0; zeros(4,1);   zeros(28,1)]; 
ub=[PU; 10; ones(4,1)*.5; ones(28,1)*100];
% start point
pz0=[C+0.2; 3.74; .2;.2;.2;.2; zeros(28,1);];
pz0(8)=1.99;
p_init(15)=3.74;

tic;
options = optimset('Display','iter','Algorithm','active-set', ...
     'TolFun',TolOverall,'TolCon',TolOverall,'TolX',TolOverall);
[pzsol,fval,exitflag]=fmincon(@objfun,pz0,[],[],[],[],lb,ub,@focfun,options);
cpu_time=toc;

p=pzsol(1:15);     % #15 is the price of the new product
pnew=pzsol(15);
znew=pzsol(16:19); % the new product attributes
z=[Z;znew'];
c=sum(z(:,1)*10*0.3+z(:,2)*10*0.8+z(:,3)*10*0.4+z(:,4)*10*0.2,2);

s=gets(p,z);
demand=sum(s,1)';
mktshr=sum(s,1)'/I;
profit=(p-c).*demand;

disp('brand  $ cost  p_init  $price  %share  demand  profit');
disp('-----  ------  ------  ------  ------  ------  ------');
for j=1:J
  disp(sprintf('%2d%3s  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f',...
    j,brand(j,1:3),c(j),p_init(j),p(j),mktshr(j)*100,demand(j),profit(j)));
end
disp('-----------------------------------------------------');
disp('The solution for Stackbelberg game scenario:');
disp(sprintf('Optimal price of new product (#15) = $%6.3f',pnew));
disp(sprintf('Aspirin  = %7.3f mg',znew(1)*1e3));
disp(sprintf('Asp-sub  = %7.3f mg',znew(2)*1e3));
disp(sprintf('Caffeine = %7.3f mg',znew(3)*1e3));
disp(sprintf('Add ingd = %7.3f mg',znew(4)*1e3));

options = optimset('Display','off','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);
% Verify Nash of each product
disp('=== start to do Nash check for each product ===');
for j=1:(J-1) % verify all competing products (except J: new product)
    pj0=c(j);
    [pjchk,nfval,exitflag]=fmincon(@objchk,pj0,[],[],[],[],c(j),3*c(j),[],options,pzsol,j);
    profitchk_j=-1*nfval;
    %disp([profitchk_j, profit(j)]);
    if profitchk_j > profit(j)
        disp(sprintf('Nash check for j=%2d%-3s failed (%-12.10f >  %-12.10f)',...
            j,brand(j,1:3),profitchk_j,profit(j)));
    else
        disp(sprintf('Nash check for j=%2d%-3s passed (%-12.10f <= %-12.10f)',...
            j,brand(j,1:3),profitchk_j,profit(j)));
    end
end

% normaliz first and calculate solution error
pz_max=[10*ones(15,1); .5*ones(4,1)];
pz_min=zeros(19,1);
norm_inf=max( abs(pzsol(1:19)-pzsol_stkg_true)./(pz_max-pz_min) );
norm_euc=sqrt(sum(( (pzsol(1:19)-pzsol_stkg_true)./(pz_max-pz_min)).^2));
disp(sprintf('convergence tolerance = %-0.1e',TolOverall));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f',cpu_time));