* Show the original market situation "before" new product entry
* Just calcualtion - no solving process
* filename: grinder_0a.gms
* Demand model: latent class model from Lan Lou's disseration (2005)
*
* Code by Ching-Shin (Norman) Shiau
* Date: Nov. 30, 2007

$Oneolcom

sets
  seg      market segment /seg1*seg4/,
  j        exisitng products in market /A,B,C/,
  atts     attributes /price,amps,life,switch,girth/,
  amps     level of current ratings /6,9,12/,
  life     level of product life /80,120,150/,
  switch   type of swtich /PD,TS,SS,TG/,
  girth type of girth /S,L/;
alias (jj,j);

parameters
  Q      total market size /9/, !! 9 million: data from Lou's paper (2007)
  mktsize(seg) size of market segment
          / seg1=0.378, seg2=0.248, seg3=0.121, seg4=0.253 /,
  a2(seg) a2 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 0.0020443, seg2=0.0026623,
            seg3=-0.0044753, seg4=0.0005643 /,
  a1(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1=-0.4029413, seg2=-0.5270453,
            seg3= 0.8923093, seg4=-0.1119013 /,
  a0(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 18.962681, seg2=24.929960,
            seg3=-42.556882, seg4= 5.303201 /,
  pw_no(seg) partworth of outside good
          / seg1=-0.022, seg2=-0.022, seg3=-0.022, seg4=-0.022 /,
*  c(j) unit cost of product /A=68.15,B=100.94,C=49.58/, !! cost data from Lou's paper (2007)
  c(j)           assume zero cost /A=0,B=0,C=0/,
  vcpt(seg,j)    utiliy of competing products,
  mktshr(seg,j)  market shares of competing products,
  profit(j)      profits of competing products,
  shrno(seg)     market share of no-purchase option (outside good);

table pw_brand(j,seg) part-worth of brand attribute
      seg1     seg2    seg3     seg4
A     0.183    1.063  -2.372   -0.203
B     0.832    0.112  -1.589    1.153
C    -0.470   -1.629   1.741   -0.785;

table pw_amps(amps,seg) part-worth of current rating attribute
      seg1     seg2    seg3    seg4
6     1.254    0.453  -1.481  -0.457
9     0.132   -1.422  -0.653  -2.381
12   -1.386    0.969   2.134   2.838;

table pw_life(life,seg) part-worth of product life attribute
      seg1     seg2    seg3    seg4
80   -0.863   -0.127  -4.717   0.802
120   1.337   -0.471  -5.825   0.743
150  -0.474    0.598  10.542  -1.545;

table pw_switch(switch,seg) part-worth of switch attribute
      seg1     seg2    seg3    seg4
PD    0.428    0.299  -3.291  -0.651
TS   -1.015   -0.653  -3.045   0.415
SS    2.392   -0.073   2.463   0.563
TG   -1.805    0.427   3.873  -0.327;

table pw_girth(girth,seg) part-worth of girth attribute
      seg1     seg2    seg3     seg4
S     1.51     0.718   1.512   0.415
L    -1.51    -0.718  -1.512  -0.415;

* price and attributes of competing products
table z(atts,j)
          A      B      C
price     99     129    79
amps      9      12     6
life      120    150    80
switch    3      1      1
girth     2      1      1;

* calculate utility for the competiting products in the market
vcpt(seg,j) = pw_brand(j,seg)  !! partworth of brand
     +a2(seg)*z('price',j)*z('price',j)+a1(seg)*z('price',j)+a0(seg)
     +pw_amps('6',seg)$(z('amps',j)=6)+pw_amps('9',seg)$(z('amps',j)=9)
         +pw_amps('12',seg)$(z('amps',j)=12)
     +pw_life('80',seg)$(z('life',j)=80)+pw_life('120',seg)$(z('life',j)=120)
         +pw_life('150',seg)$(z('life',j)=150)
     +pw_switch('PD',seg)$(z('switch',j)=1)+pw_switch('TS',seg)$(z('switch',j)=2)
     +pw_switch('SS',seg)$(z('switch',j)=3)+pw_switch('TG',seg)$(z('switch',j)=4)
     +pw_girth('S',seg)$(z('girth',j)=1)+pw_girth('L',seg)$(z('girth',j)=2);

* post-process and display
mktshr(seg,j) = exp(vcpt(seg,j))/(exp(pw_no(seg))+sum(jj,exp(vcpt(seg,jj))));
shrno(seg) = exp(pw_no(seg))/(exp(pw_no(seg))+sum(jj,exp(vcpt(seg,jj))));
profit(j) = Q*sum(seg,mktsize(seg)*mktshr(seg,j))*(z('price',j)-c(j));

file log / '' /;
put log;
put ' ' /;
put 'Market situation before new product entry' /;
put ' ' /;
put 'Product Price$ cost$ MktShr% Profit$  Amps  Life  Switch  girth' /;
put '---------------------------------------------------------------' /;
loop(j,
     put '   ',j.tl:1:0,(z('price',j)):9:1, c(j):7:1,
     (sum(seg,100*mktsize(seg)*mktshr(seg,j))):7:2,(profit(j)):6:0,'m'
     z('amps',j):7:0,z('life',j):7:0,z('switch',j):6:0,z('girth',j):8:0 /;
     );
put '---------------------------------------------------------------' /;
put 'Switch type: 1=Paddle 2=Top-Slider 3=Side-Slider 4=Trigger' /;
put ' Girth type: 1=Small  2=Large' /;
put ' ' /;
put '       ',(mktsize('seg1')*100):7:2,'%',(mktsize('seg2')*100):7:2,'%',
              (mktsize('seg3')*100):7:2,'%',(mktsize('seg4')*100):7:2,'%',
              (sum(seg,mktsize(seg)*100)):7:2,'%' /;
put 'Product  Seg 1   Seg 2   Seg 3   Seg 4   Total%' /;
put '-----------------------------------------------' /;
loop(j,
     put '   ',j.tl:1:0,'  ',(mktshr('seg1',j)*100):8:2,(mktshr('seg2',j)*100):8:2,
     (mktshr('seg3',j)*100):8:2,(mktshr('seg4',j)*100):8:2,
     (sum(seg,100*mktsize(seg)*mktshr(seg,j))):8:2 /;
    );
put 'Not buy',(shrno('seg1')*100):7:2,(shrno('seg2')*100):8:2,
   (shrno('seg3')*100):8:2,(shrno('seg4')*100):8:2,
   (sum(seg,mktsize(seg)*shrno(seg)*100)):8:2 /;
put '-----------------------------------------------' /;
put ' ' /;
putclose;