* Product positioning for angle grinder (class I)
* filename: grinder_1a.gms
*
* Demand model: latent class model from Lan Lou's disseration (2005)
* Variable: price (continuous)
*           amps, life, switch and actuator (discrete)
*
* This code is to optimize new product with prices and attributes
*   of competing products fixed (solution of a class I problem)
*
* Code by Ching-Shin (Norman) Shiau
* Date: Nov. 29, 2007

$Oneolcom

sets
  seg      market segment /seg1*seg4/,
  j        exisitng products in market /A,B,C/,
  atts     attributes /price,amps,life,switch,girth/,
  amps     level of current ratings /6,9,12/,
  life     level of product life /80,110,150/,
  switch   type of swtich /PD,TS,SS,TG/,
  girth type of girth /S,L/;
alias (jj,j);

parameters
  Q      total market size /9/, !! 9 million: data from Lou's paper (2007)
  segratio(seg) ratio of market segment
          / seg1=0.378, seg2=0.248, seg3=0.121, seg4=0.253 /,
  mktsize(seg)  size of market segment,
  a2(seg) a2 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 0.00204433333333, seg2=0.00266233333333,
            seg3=-0.00447533333333, seg4=0.00056433333333 /,
  a1(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1=-0.40294133333333, seg2=-0.52704533333333,
            seg3= 0.89230933333333, seg4=-0.11190133333333 /,
  a0(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 18.96268099999990, seg2=24.92995899999988,
            seg3=-42.55688199999980, seg4= 5.30320099999997 /,
  pw_no(seg) partworth of outside good
          / seg1=-0.022, seg2=-0.022, seg3=-0.022, seg4=-0.022 /,
  pw_bx(seg) partworth for brand X
          / seg1=-0.545, seg2=0.454, seg3=2.211, seg4=-0.165/,
*  c(j) unit cost of product /A=70.74,B=103.62,C=51.08/, !! cost data from Lou's thesis (2007)
  c(j)  /A=68.15,B=100.94,C=49.58/,
  cnew /75/,
  vcpt(seg,j)    utiliy of competing products,
  mktshr(seg,j)  market shares of competing products,
  profit(j)      profits of competing products,
  shrno(seg)     market share of no-chocie,
  shrnew(seg)    market share of new product;

mktsize(seg)=Q*segratio(seg);

table pw_brand(j,seg) part-worth of brand attribute
      seg1     seg2    seg3     seg4
A     0.183    1.063  -2.372   -0.203
B     0.832    0.112  -1.589    1.153
C    -0.470   -1.629   1.741   -0.785;

table pw_amps(amps,seg) part-worth of current rating attribute
      seg1     seg2    seg3    seg4
6     1.254    0.453  -1.481  -0.457
9     0.132   -1.422  -0.653  -2.381
12   -1.386    0.969   2.134   2.838;

table pw_life(life,seg) part-worth of product life attribute
      seg1     seg2    seg3    seg4
80   -0.863   -0.127  -4.717   0.802
110   1.337   -0.471  -5.825   0.743
150  -0.474    0.598  10.542  -1.545;

table pw_switch(switch,seg) part-worth of switch attribute
      seg1     seg2    seg3    seg4
PD    0.428    0.299  -3.291  -0.651
TS   -1.015   -0.653  -3.045   0.415
SS    2.392   -0.073   2.463   0.563
TG   -1.805    0.427   3.873  -0.327;

table pw_girth(girth,seg) part-worth of girth attribute
      seg1     seg2    seg3     seg4
S     1.51     0.718   1.512   0.415
L    -1.51    -0.718  -1.512  -0.415;

* price and attributes of competing products
table z(atts,j)
          A      B      C
price     99     129    79
amps      9      12     6
life      110    150    80
switch    3      1      1
girth     2      1      1;

* calculate utility for the competiting products in the market
vcpt(seg,j) = pw_brand(j,seg)  !! partworth of brand
     +a2(seg)*z('price',j)*z('price',j)+a1(seg)*z('price',j)+a0(seg)
     +pw_amps('6',seg)$(z('amps',j)=6)+pw_amps('9',seg)$(z('amps',j)=9)
         +pw_amps('12',seg)$(z('amps',j)=12)
     +pw_life('80',seg)$(z('life',j)=80)+pw_life('110',seg)$(z('life',j)=110)
         +pw_life('150',seg)$(z('life',j)=150)
     +pw_switch('PD',seg)$(z('switch',j)=1)+pw_switch('TS',seg)$(z('switch',j)=2)
     +pw_switch('SS',seg)$(z('switch',j)=3)+pw_switch('TG',seg)$(z('switch',j)=4)
     +pw_girth('S',seg)$(z('girth',j)=1)+pw_girth('L',seg)$(z('girth',j)=2);

*variable pnew,vnew(seg),vcpt(seg,j),snew;
variable vnew(seg),pinew;
positive variables pnew;
binary variables y1(amps),y2(life),y3(switch),y4(girth);

equations  sum_y1,sum_y2,sum_y3,sum_y4,fun_vnew(seg),fun_profit;

* constraint for binary choices
sum_y1 .. sum(amps    ,y1(amps))     =e= 1;
sum_y2 .. sum(life    ,y2(life))     =e= 1;
sum_y3 .. sum(switch  ,y3(switch))   =e= 1;
sum_y4 .. sum(girth,y4(girth)) =e= 1;

* calculate utility for the new product
fun_vnew(seg) ..  vnew(seg) =e=   pw_bx(seg)  !! partworh of brand X
     +a2(seg)*pnew*pnew+a1(seg)*pnew+a0(seg)  !! partworth of price
     +sum(amps,    pw_amps(amps,seg)*y1(amps) )
     +sum(life,    pw_life(life,seg)*y2(life) )
     +sum(switch,  pw_switch(switch,seg)*y3(switch) )
     +sum(girth,pw_girth(girth,seg)*y4(girth) );

* calculate logit probability for new product
fun_profit ..  pinew =e= (pnew-cnew)*sum(seg, mktsize(seg)*
   ( exp(vnew(seg))/(exp(pw_no(seg))+exp(vnew(seg))+sum(j,exp(vcpt(seg,j))))));

vnew.l(seg)=0.1;
vnew.lo(seg)=-20;
vnew.up(seg)=20;
pnew.lo=75;
pnew.up=130;
pnew.l=100;
y1.l(amps)=0;
y2.l(life)=0;
y3.l(switch)=0;
y4.l(girth)=0;

model grinder /all/;
option minlp=dicopt;
solve grinder using minlp maximizing pinew;

* post-process and display
shrnew(seg) = exp(vnew.l(seg))/(exp(pw_no(seg))+exp(vnew.l(seg))+sum(jj,exp(vcpt(seg,jj))));
mktshr(seg,j) =  exp(vcpt(seg,j))/(exp(pw_no(seg))+exp(vnew.l(seg))+sum(jj,exp(vcpt(seg,jj))));
shrno(seg) = 1-sum(j,mktshr(seg,j))-shrnew(seg);
profit(j) = sum(seg,mktshr(seg,j)*mktsize(seg))*(z('price',j)-c(j));

file log / '' /;
put log;
put 'Class I: Optimize new product with fixed competitors' /;
put /;
put 'Product Price$ Cost $ MktShr% Profit$  Amps  Life  Switch  girth' /;
put '------- ------ ------ ------- -------  ----  ----  ------  -----' /;
loop(j,
     put '   ',j.tl:1:0,(z('price',j)):9:1,c(j):7:1,
     (sum(seg,100*segratio(seg)*mktshr(seg,j))):8:2,(profit(j)):6:0,'m'
     z('amps',j):7:0,z('life',j):7:0,z('switch',j):6:0,z('girth',j):8:0 /;
     );
put '  New',(pnew.l):8:1, (cnew):7:1,
    sum(seg,(segratio(seg)*shrnew(seg)*100)):8:2, (pinew.l):6:0,'m',
    ( 6$(y1.l('6')=1)+9$(y1.l('9')=1)+12$(y1.l('12')=1)):7:0,
    ( 80$(y2.l('80')=1)+110$(y2.l('110')=1)+150$(y2.l('150')=1)):7:0,
    ( 1$(y3.l('PD')=1)+2$(y3.l('TS')=1)+3$(y3.l('SS')=1)+4$(y3.l('TG')=1) ):6:0,
    ( 1$(y4.l('S')=1)+2$(y4.l('L')=1) ):8:0 /;
put '-------' /;
put 'Switch type = 1:Paddle/2:Top-Slider/3:Side-Slider/4:Trigger' /;
put 'Girth  type = 1:Small/2:Large' /;
put /;
put '       ',(segratio('seg1')*100):7:2,'%',(segratio('seg2')*100):7:2,'%',
              (segratio('seg3')*100):7:2,'%',(segratio('seg4')*100):7:2,'%',
              (sum(seg,segratio(seg)*100)):7:2,'%' /;
put 'Product  Seg 1   Seg 2   Seg 3   Seg 4   Total%' /;
put '-------  ------  ------  ------  ------  ------' /;
loop(j,
     put '   ',j.tl:1:0,'  ',(mktshr('seg1',j)*100):8:2,(mktshr('seg2',j)*100):8:2,
     (mktshr('seg3',j)*100):8:2,(mktshr('seg4',j)*100):8:2,
     (sum(seg,100*segratio(seg)*mktshr(seg,j))):8:2 /;
    );
put '  New ',(shrnew('seg1')*100):8:2,(shrnew('seg2')*100):8:2,
    (shrnew('seg3')*100):8:2,(shrnew('seg4')*100):8:2,
    (sum(seg,(segratio(seg)*shrnew(seg)*100))):8:2  /;
put 'Not buy',(shrno('seg1')*100):7:2,(shrno('seg2')*100):8:2,
   (shrno('seg3')*100):8:2,(shrno('seg4')*100):8:2,
   (sum(seg,segratio(seg)*shrno(seg)*100)):8:2 /;
putclose;