$Title  Calculate the market equilibrium 'after' new product entry

* filename: grinder_1b_lgn.gms  (for realized profit)
* Assume new product is fixed at pre-optimal deisgn attributes
* [(3)12 amps, (2)110 hours, (3)Side Slider, (1)Small Girth]
*  then let everyone (including) do Nash competition to reach NPE
*
* Demand model: latent class model from Lan Lou's disseration (2005)
* add price upper bound and Lagragian multiplier
*
* Code by Ching-Shin (Norman) Shiau
* Date: Dec. 7, 2007

$oneolcom

sets
  seg      market segment /seg1*seg4/,
  j        exisitng products in market /A,B,C/,
  atts     attributes /amps,life,switch,girth/,
  amps     level of current ratings /6,9,12/,
  life     level of product life /80,110,150/,
  switch   type of swtich /PD,TS,SS,TG/,
  girth    type of girth /S,L/;
alias (jj,j);

parameters
      Q      total market size /9/, !! 9 million: data from Lou's paper (2007)
  segratio(seg) ratio of market segment
          / seg1=0.378, seg2=0.248, seg3=0.121, seg4=0.253 /,
  a2(seg) a2 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 0.0020443, seg2=0.0026623,
            seg3=-0.0044753, seg4=0.0005643 /,
  a1(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1=-0.4029413, seg2=-0.5270453,
            seg3= 0.8923093, seg4=-0.1119013 /,
  a0(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 18.962681, seg2=24.929959,
            seg3=-42.556882, seg4= 5.303201 /,
  pw_no(seg) partworth of outside good (no-purchase option)
          / seg1=-0.022, seg2=-0.022, seg3=-0.022, seg4=-0.022 /,
  pw_bx(seg) partworth for brand X (new product)
          / seg1=-0.545, seg2=0.454, seg3=2.211, seg4=-0.165/,
*  c(j) unit cost of product /A=68.15,B=100.94,C=49.58/, !! cost data from Lou's paper (2007)
  c(j)  /A=68.15,B=100.94,C=49.58/,
  p_init(j) / A=99,B=129,C=79 /,
  vz(seg,j)      utility of competing products on design attributes z,
  vnew(seg)      utility of new product fixed,
  pinew          profit of new product,
  mktshr(seg,j)  market shares of competing products,
  profit(j)      profits of competing products,
  shrnew(seg)    market share of the new product,
  shrno(seg)     market share of no-purchase option (outside good),
  mktsize(seg)  size of market segment;

mktsize(seg)=Q*segratio(seg);

table pw_brand(j,seg) part-worth of brand attribute
      seg1     seg2    seg3     seg4
A     0.183    1.063  -2.372   -0.203
B     0.832    0.112  -1.589    1.153
C    -0.470   -1.629   1.741   -0.785;

table pw_amps(amps,seg) part-worth of current rating attribute
      seg1     seg2    seg3    seg4
6     1.254    0.453  -1.481  -0.457
9     0.132   -1.422  -0.653  -2.381
12   -1.386    0.969   2.134   2.838;

table pw_life(life,seg) part-worth of product life attribute
      seg1     seg2    seg3    seg4
80   -0.863   -0.127  -4.717   0.802
110   1.337   -0.471  -5.825   0.743
150  -0.474    0.598  10.542  -1.545;

table pw_switch(switch,seg) part-worth of switch attribute
      seg1     seg2    seg3    seg4
PD    0.428    0.299  -3.291  -0.651
TS   -1.015   -0.653  -3.045   0.415
SS    2.392   -0.073   2.463   0.563
TG   -1.805    0.427   3.873  -0.327;

table pw_girth(girth,seg) part-worth of girth attribute
      seg1     seg2    seg3     seg4
S     1.51     0.718   1.512   0.415
L    -1.51    -0.718  -1.512  -0.415;

* price and attributes of competing products
table z(atts,j)
          A    B    C
amps      9    12   6
life      110  150  80
switch    3    1    1
girth     2    1    1;

* calculate utility for the competiting products in the market
vz(seg,j) = pw_brand(j,seg)  !! partworth of brand
*     +a2(seg)*z('price',j)*z('price',j)+a1(seg)*z('price',j)+a0(seg)
     +pw_amps('6',seg)$(z('amps',j)=6)+pw_amps('9',seg)$(z('amps',j)=9)
        +pw_amps('12',seg)$(z('amps',j)=12)
     +pw_life('80',seg)$(z('life',j)=80)+pw_life('110',seg)$(z('life',j)=110)
        +pw_life('150',seg)$(z('life',j)=150)
     +pw_switch('PD',seg)$(z('switch',j)=1)+pw_switch('TS',seg)$(z('switch',j)=2)
        +pw_switch('SS',seg)$(z('switch',j)=3)+pw_switch('TG',seg)$(z('switch',j)=4)
     +pw_girth('S',seg)$(z('girth',j)=1)+pw_girth('L',seg)$(z('girth',j)=2);

vnew(seg) = pw_bx(seg)  !! partworth of brand
             +a2(seg)*130**2+a1(seg)*130+a0(seg)   !! optimal price = $130
             +pw_amps('12',seg)   +  pw_life('110',seg)
             +pw_switch('SS',seg) +  pw_girth('S',seg);

variables v(seg,j),u;
positive variables p(j),s(seg,j),mu(j);
equations f_v(seg,j),f_s(seg,j),f_obj,f_foc(j),f_mug(j);

* calculate logit probability for competing products
f_v(seg,j) .. v(seg,j) =e=
        vz(seg,j)+ a2(seg)*p(j)*p(j)+a1(seg)*p(j)+a0(seg);

f_s(seg,j) .. s(seg,j) =e=  exp(v(seg,j))
      /( exp(pw_no(seg))+ exp(vnew(seg)) + sum(jj,exp(v(seg,jj))) );

* FOC equation of price equilibrium for competing products j
f_foc(j) .. sum(seg, mktsize(seg)*s(seg,j)
      *( (2*a2(seg)*p(j)+a1(seg))*(1-s(seg,j))*(p(j)-c(j))+1 ) ) - mu(j) =e= 0;

f_obj .. u =e= 0;

p.up(j)=130;
f_mug(j) .. mu(j)*(p(j)-p.up(j)) =e= 0;

mu.l(j)=10;
p.l(j)=130;
v.l(seg,j)=vz(seg,j)+ a2(seg)*p.l(j)*p.l(j)+a1(seg)*p.l(j)+a0(seg);
v.lo(seg,j)=-30;
v.up(seg,j)=30;
s.l(seg,j)=exp(v.l(seg,j))/( exp(pw_no(seg))+ exp(vnew(seg)) + sum(jj,exp(v.l(seg,jj))) );
s.lo(seg,j)=0;
s.up(seg,j)=1;

model grinder /all/;
option nlp=conopt;
solve grinder using nlp maximizing u;

* post-process and display
shrnew(seg) =  exp(vnew(seg)) /( exp(pw_no(seg))+ exp(vnew(seg)) + sum(jj,exp(v.l(seg,jj))) );
shrno(seg) = 1-sum(j,s.l(seg,j))-shrnew(seg);
pinew = sum(seg, mktsize(seg)*shrnew(seg) )*(130-75);
profit(j) = sum(seg,mktsize(seg)*s.l(seg,j))*(p.l(j)-c(j));

file log / '' /;
put log;
put ' ' /;
put 'Nash price equilibrium after new product entry' /;
put ' ' /;
put 'Product P_init$ P_solu$  cost$  MktShr% Profit$  Amps  Life  Switch  girth' /;
put '--------------------------------------------------------------------------' /;
loop(j,
     put '   ',j.tl:1:0,(p_init(j)):10:1,(p.l(j)):8:1,c(j):8:1,
     (sum(seg,100*segratio(seg)*s.l(seg,j))):8:2,(profit(j)):6:0,'m'
     z('amps',j):7:0,z('life',j):7:0,z('switch',j):6:0,z('girth',j):8:0 /;
     );
put '  New',(130):9:1, (130):8:1, (75):8:1,
    sum(seg,(segratio(seg)*shrnew(seg)*100)):8:2, (pinew):6:0,'m',
    (12):7:0, (110):7:0, (3):6:0, (1):8:0 /;
put '--------------------------------------------------------------------------' /;
put 'Switch type: 1=Paddle 2=Top-Slider 3=Side-Slider 4=Trigger' /;
put ' Girth type: 1=Small  2=Large' /;
put ' '/;
put '       ',(segratio('seg1')*100):7:2,'%',(segratio('seg2')*100):7:2,'%',
              (segratio('seg3')*100):7:2,'%',(segratio('seg4')*100):7:2,'%',
              (sum(seg,segratio(seg)*100)):7:2,'%' /;
put 'Product  Seg 1   Seg 2   Seg 3   Seg 4   Total%' /;
put '-----------------------------------------------' /;
loop(j,
     put '   ',j.tl:1:0,'  ',(s.l('seg1',j)*100):8:2,(s.l('seg2',j)*100):8:2,
     (s.l('seg3',j)*100):8:2,(s.l('seg4',j)*100):8:2,
     (sum(seg,100*segratio(seg)*s.l(seg,j))):8:2 /;
    );
put '   New',(shrnew('seg1')*100):8:2,(shrnew('seg2')*100):8:2,
     (shrnew('seg3')*100):8:2,(shrnew('seg4')*100):8:2,
     (sum(seg,100*segratio(seg)*shrnew(seg))):8:2 /;
put 'Not buy',(shrno('seg1')*100):7:2,(shrno('seg2')*100):8:2,
   (shrno('seg3')*100):8:2,(shrno('seg4')*100):8:2,
   (sum(seg,segratio(seg)*shrno(seg)*100)):8:2 /;
put '-----------------------------------------------' /;
put ' ' /;
putclose;