% code for class II: optimal design under price equilibrium (foc=0)
%
% Case study from:
% Jeremy J. Michalek, Fred M. Feinberg, and Panos Y. Papalambros
% "Linking Marketing and Engineering Product Design Decisions via
% Analytical Target Cascading" J PROD INNOV MANAG 2005, 22:42, V62
%
% Code: Ching-Shin Norman Shiau
% Date: Feb. 7, 2008
% Design Decisions Laboratory
% Dept. of Mechanical Engineering
% Carnegie Mellon University
% Pittsburgh, PA

format compact
format long
clear all
close all
clc

global P Z Q y c cf J R GP t
global LCSegmentSize p_min z_min p_max z_max betaNCLC zcf
load '../latent_class_data.mat'
load 'nash_true.mat'

TolOverall=1e-7; % best: 5e-8
t=1e-9; % relaxation parameter in Reg(t)
J=4; % # of existing products in market
brand=['C1 ';'R2 ';'S3 ';'T4 ';'New'];
% #      z1   z2    z3   z4    z5    
Z(1,:)=[350  1.02  120  0.188  1.40]; % C1  
Z(2,:)=[250  0.86  105  0.094  1.25]; % R2 
Z(3,:)=[280  0.89  136  0.156  1.70]; % S3 
Z(4,:)=[320  1.06  115  0.125  1.15]; % T4 
P0=[29.99; 19.99; 25.95; 22.95];

% parameters
Q=5;    % total market size /5/,  !! 5 million units
c=3;    % unit cost (constant) /3/,
cf=1;   % fixed (investment) cost /1/,  !! $1 million
y(1)=0.3;   % Gap between base and cover (inch) /0.30/,
y(2)=0.5;   % Minimum distance between spring and base (inch) /0.50/,
y(3)=1.90;  % Internal thickness of scale (inch) /1.90/,
y(4)=0.25;  % Minimum pinion pitch diameter (inch) /0.25/,
y(5)=3.0;   % Length of window (inch) /3.0/,
y(6)=2.0;   % Width of window (inch) /2.0/,
y(7)=1.13;  % Distance between top of cover and window (inch) /1.13/,
y(8)=1.0;   % Number of lbs measured per tick mark (lbs) /1.0/,
y(9)=1.10;  % Horizontal dist. spring to pivot (inch) /1.10/,
y(10)=0.31; % Length of tick mark gap to number (inch) /0.31/,
y(11)=16.0; % Number of lbs that number spans (lbs) /16.0/,
y(12)=1.29; % Aspect ratio of number length-to-width /1.29/,
y(13)=4.0;  % Min. allow lever dist. base to centerline (in) /4.0/,

% variables
%  x1  Length from Base to Force on Long Lever,
%  x2  Length from Force to Spring on Long Lever,
%  x3  Length from Base to Force on Short Lever,
%  x4  Length from Force to Joint on Short Lever,
%  x5  Length from Force to Joint on Long Lever,
%  x6  Spring Constant,
%  x7  Distance from Base Edge to Spring,
%  x8  Length of Rack,
%  x9  Pitch Diameter of Pinion,
%  x10 Length of Pivot Horizontal Arm,
%  x11 Length of Pivot Vertical Arm,
%  x12 Dial Diameter,
%  x13 Cover Length,
%  x14 Cover Width,
lb=[0.125;0.125;0.125;0.125;0.125;  1;0.5; 1;0.25;0.5;0.5; 9; 9; 9;...
    10*ones(J+1,1); zeros(18+28+10,1)];
ub=[   36;   36;   24;   24;   36;200; 12;36;  24;1.9;1.9;13;13;13;...
    30*ones(J+1,1); 10*ones(18+28+10,1)];

options = optimset('Display','iter','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);
SolCount=0;

% multi-start loop
% for pnew=14:3:17
%    for p1=14:3:17
%      for p2=14:3:17
%        for p3=14:3:17
%          for p4=14:3:17
%     pp0=[pnew,p1,p2,p3,p4];  
    
xp0=[4.6; 7.3; 4.6; 2.3; 1.2;... %x1-5
     90; 0.5; 6.83; 0.35; 0.8; ... %x6-10
     1.7; 9.9; 12.1; 11.6;...  %x11-14
     16.7; 17; 15; 18; 18;... % pnew/p1-p4
     zeros(18+28+10,1);]; % mu1-mu56

tic;
    [xpsol,nfval,exitflag1]=fmincon(@objfun,xp0,[],[],[],[],lb,ub,@confun,options);
cpu_time = toc;

    if exitflag1 >= 1
        SolCount=SolCount+1;
        sols(SolCount,:)=xpsol;
        psol=xpsol(15:19);
        zsol=getz(xpsol(1:14));
        zpsol(:,SolCount)=[zsol;psol];
    end
     
%         end
%       end
%     end
%   end
% end
% end of multi-start loop

save FOCSOLS.mat sols

% check Nash condition
disp('===== Start of Nash Verification =====');
VerifyNash

% normaliz first and calculate solution error
zp_min=[200;0.75;100;2/32;0.75;10;10;10;10;10];
zp_max=[400;1.333;140;6/32;1.75;30;30;30;30;30];
norm_inf=max( abs(zpsol-zpsol_nash_true)./(zp_max-zp_min) );
norm_euc=sqrt(sum(( (zpsol-zpsol_nash_true)./(zp_max-zp_min)).^2));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f',cpu_time));