% code for class II: optimal design under price equilibrium (foc=0)
%
% Case study from:
% Jeremy J. Michalek, Fred M. Feinberg, and Panos Y. Papalambros
% "Linking Marketing and Engineering Product Design Decisions via
% Analytical Target Cascading" J PROD INNOV MANAG 2005, 22:42, V62
%
% Code: Ching-Shin Norman Shiau
% Date: Feb. 7, 2008
% Design Decisions Laboratory
% Dept. of Mechanical Engineering
% Carnegie Mellon University
% Pittsburgh, PA

format compact
format short
clear all
close all
clc

global P Z Q y c cf J R GP follower
global LCSegmentSize p_min z_min p_max z_max betaNCLC zcf
load '../latent_class_data.mat'
load pzstkg_sols_true.mat

TolOverall=1e-2;
J=4; % # of existing products in market
brand=['C1 ';'R2 ';'S3 ';'T4 ';'New'];
% #      z1   z2    z3   z4    z5    
Z(1,:)=[350  1.02  120  0.188  1.40]; % C1  
Z(2,:)=[250  0.86  105  0.094  1.25]; % R2 
Z(3,:)=[280  0.89  136  0.156  1.70]; % S3 
Z(4,:)=[320  1.06  115  0.125  1.15]; % T4 
P0=[29.99; 19.99; 25.95; 22.95];

% parameters
Q=5;    % total market size /5/,  !! 5 million units
c=3;    % unit cost (constant) /3/,
cf=1;   % fixed (investment) cost /1/,  !! $1 million
y(1)=0.3;   % Gap between base and cover (inch) /0.30/,
y(2)=0.5;   % Minimum distance between spring and base (inch) /0.50/,
y(3)=1.90;  % Internal thickness of scale (inch) /1.90/,
y(4)=0.25;  % Minimum pinion pitch diameter (inch) /0.25/,
y(5)=3.0;   % Length of window (inch) /3.0/,
y(6)=2.0;   % Width of window (inch) /2.0/,
y(7)=1.13;  % Distance between top of cover and window (inch) /1.13/,
y(8)=1.0;   % Number of lbs measured per tick mark (lbs) /1.0/,
y(9)=1.10;  % Horizontal dist. spring to pivot (inch) /1.10/,
y(10)=0.31; % Length of tick mark gap to number (inch) /0.31/,
y(11)=16.0; % Number of lbs that number spans (lbs) /16.0/,
y(12)=1.29; % Aspect ratio of number length-to-width /1.29/,
y(13)=4.0;  % Min. allow lever dist. base to centerline (in) /4.0/,

% variables
%  x1  Length from Base to Force on Long Lever,
%  x2  Length from Force to Spring on Long Lever,
%  x3  Length from Base to Force on Short Lever,
%  x4  Length from Force to Joint on Short Lever,
%  x5  Length from Force to Joint on Long Lever,
%  x6  Spring Constant,
%  x7  Distance from Base Edge to Spring,
%  x8  Length of Rack,
%  x9  Pitch Diameter of Pinion,
%  x10 Length of Pivot Horizontal Arm,
%  x11 Length of Pivot Vertical Arm,
%  x12 Dial Diameter,
%  x13 Cover Length,
%  x14 Cover Width,
lb=[0.125,0.125,0.125,0.125,0.125,  1,0.5, 1,0.25,0.5,0.5, 9, 9, 9,10];
ub=[   36,   36,   24,   24,   36,200, 12,36,  24,1.9,1.9,13,13,13,30];
% 1-14:x1-x14, 15:pnew, 16:15+J:P
% xpnew0=[3.835  7.945  3.876  2.553  0.183  86.90  0.500...
%      6.830  0.314  0.747  1.846  9.869 12.069 11.600 17];
xpnew0=[ 4.19, 7.61,4.17,2.17,0.41...
         86.82,0.5,6.83,0.31,0.77...
         1.65,9.86,12.06,11.61 16];
 
SolCount=0;

xpall_new=zeros(1,15+J);
follower.p=[17 15 18 18];
% optimization for new product design and price
options = optimset('Display','iter','Algorithm','active-set',...
        'TolFun',TolOverall,'TolX',TolOverall);
tic;
[xpnewsol,nfval,exitflag1]=fmincon(@objfunnew,xpnew0,...
                [],[],[],[],lb,ub,@confun,options);
cpu_time=toc;

if exitflag1 >0
    xpall_new(1:15)=xpnewsol; % update
else
    disp(sprintf('warning: exitflag1 = %d',exitflag1)); % not update
end
   
sols=xpall_new;
save FOCSOLS.mat sols

% check Nash condition
disp('===== Start of Nash Verification =====');
VerifyNash

znew=getz(xpsol(1:14));
pzsol=[ xpsol(15:19)';znew'];

% normaliz first and calculate solution error
pz_min=[10*ones(5,1); 200; 0.75; 100; 0.063; 0.75;];
pz_max=[30*ones(5,1); 400; 1.3333; 140; 0.1875; 1.75];
norm_inf=max( abs(pzsol - pzsol_stkg_sols)./(pz_max-pz_min) );
norm_euc=sqrt(sum(( (pzsol - pzsol_stkg_sols)./(pz_max-pz_min)).^2));
disp(sprintf('convergence tolerance = %-0.1e',TolOverall));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f',cpu_time));